#!/usr/bin/env python
"""*********************************************************************
* This file is part of Edward the GPG Bot.                             *
*                                                                      *
* Edward is free software: you can redistribute it and/or modify       *
* it under the terms of the GNU Affero Public License as published by  *
* the Free Software Foundation, either version 3 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* Edward is distributed in the hope that it will be useful,            *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU Affero Public License for more details.                          *
*                                                                      *
* You should have received a copy of the GNU Affero Public License     *
* along with Edward.  If not, see <http://www.gnu.org/licenses/>.      *
*                                                                      *
* Josh Drake, Copyright (c) 2014                                       *
* Lisa Marie Maginnis, Copyright (c) 2014                              *
*********************************************************************"""
import re
import os
import sys
import gnupg
import time
import subprocess
import gpg_filter_config as config

import locale
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')


# Init gpg enviroment
gpg = gnupg.GPG(gnupghome=config.gnupghome, keyring=config.keyring, secret_keyring=config.secret_keyring)

def debug(msg):
    if config.debug == 1:
        print "[%s] %s" % (time.asctime(), msg)

# Parses our emails
def parse(message, begin_token, end_token):
    found=0
    line="1"
    result=""

    debug("Parsing")

    for line in message:
        if re.match(begin_token, line) != None:
            found=1
            debug("parse(): beginning parsing=%s" % line)
        if found == 1:
            result = "%s%s" % (result, line)
            debug("parse(): adding line=%s" % line)
        if re.match(end_token, line) != None:
            debug("parse(): ending parsing")
            break

    return result

line = "1"
in_message = []
while line:
    line=sys.stdin.readline()
    in_message.append(line)


from_line = parse(in_message, "^From:", "^From:").rstrip()
author = "".join(from_line.split(":")[1:]).strip()
author_email = from_line.split("<")[1:][0].strip(">")
to_line = parse(in_message, "^To:", "^To:").rstrip()

# Pick the right langauge
if '-de' in to_line:
    import gpg_filter_langpack_de as lang
    our_email = 'edward-de'
elif '-el' in to_line:
    import gpg_filter_langpack_el as lang
    our_email = 'edward-el'
elif '-ja' in to_line:
    import gpg_filter_langpack_ja as lang
    our_email = 'edward-ja'
elif '-ro' in to_line:
    import gpg_filter_langpack_ro as lang
    our_email = 'edward-ro'
elif '-ru' in to_line:
    import gpg_filter_langpack_ru as lang
    our_email = 'edward-ru'
elif '-fr' in to_line:
    import gpg_filter_langpack_fr as lang
    our_email = 'edward-fr'
elif '-fr' in to_line:
    import gpg_filter_langpack_tr as lang
    our_email = 'edward-tr'
elif '-pt-br' in to_line:
    import gpg_filter_langpack_pt_br as lang
    our_email = 'edward-pt-br'
else:
    import gpg_filter_langpack_en as lang
    our_email = 'edward-en'


msg = parse(in_message, "^-----BEGIN PGP MESSAGE-----", "^-----END PGP MESSAGE-----")
key = parse(in_message, "^-----BEGIN PGP PUBLIC KEY BLOCK-----", "^-----END PGP PUBLIC KEY BLOCK-----")
sig = parse(in_message, "^-----BEGIN PGP SIGNATURE-----", "^-----END PGP SIGNATURE-----")

print msg
print key
print sig
print author
print author_email

peer_public_key = gpg.import_keys(key)
peer_fingerprint = 0

if peer_public_key:
    peer_fingerprint = peer_public_key.fingerprints[0]
else:
    public_keys = gpg.list_keys()
    for key in public_keys:
        for uid in key['uids']:
            if author_email.lower() in uid.lower():
                peer_fingerprint = key['keyid']
                break
    

plaintext_message = str(gpg.decrypt(msg))

#print peer_public_key
#print peer_fingerprint
#print plaintext_message

replay_message = ''

if msg:
    if plaintext_message:
        reply_message = lang.replies['success_decrypt'] % (plaintext_message)
    else:
        reply_message = lang.replies['failed_decrypt']
else:
    if peer_public_key and key:
        reply_message = lang.replies['public_key_received']
    else:
        reply_message = lang.replies['no_public_key']

#enable this when ready to test
#if sig:
    #verify = subprocess.check_output
#   verified = gpg.verify("\n".join(in_message))
#   if not verified:
#        reply_message += "%(sig_failure)s\n" %  lang.replies
#    else:
#        reply_message += "%(sig_success)s\n" %  lang.replies

reply_message += '\n\n--\n%s' % lang.replies['signature']

reply_message += '\n\n%s' % 'http://agpl.fsf.org/emailselfdefense.fsf.org/edward/CURRENT/edward.tar.gz'

print reply_message

if peer_fingerprint:
    reply_message = gpg.encrypt(reply_message, recipients=[peer_fingerprint], always_trust=True)

print reply_message

os.system('(echo "Subject: %s";echo;echo "%s") | /usr/sbin/sendmail -f %s@fsf.org -F"Edward, GPG Bot" -i \'%s\'' % ("Re: FSF GPG Tester", reply_message, our_email, author_email))

